/***************************************************************************
              xstickwin : simple program to move active window on screen
                             -------------------
    begin                : Sat Aug 10 15:44:19 CEST 2002
    copyright            : (C) 2002 by Philippe Brochard
    email                : hocwp@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>


/* change this default value to feet with your window manager*/
#define N_DECO    19
#define S_DECO    5



static void analyse_cmd_line_first (int argc, char **argv);
static void analyse_cmd_line (int argc, char **argv);
static void show_usage (void);
static void get_geometry (char *str_geom, int *x, int *y, int *width,
			  int *height);
static void change_gravity (void);



static char *prog_name = "";
static Display *dpy;
static char *display_string = "";
static int x = 0, y = 0;
static unsigned int w, h, border, depth;
static unsigned int n_deco = -1, s_deco = -1, w_deco = -1, e_deco = -1;
static unsigned int root_w, root_h;
static char *grav_string = "";
static Window win, root;
static int revert_to;


int
main (int argc, char **argv)
{
  analyse_cmd_line_first (argc, argv);

  dpy = XOpenDisplay (display_string);

  if (dpy == 0)
    {
      fprintf (stderr, "Can not open display\n");
      exit (1);
    }

  root = XDefaultRootWindow (dpy);
  XGetGeometry (dpy, root, &root, &x, &y, &root_w, &root_h, &border, &depth);

  XGetInputFocus (dpy, &win, &revert_to);

  XGetGeometry (dpy, win, &root, &x, &y, &w, &h, &border, &depth);
  x = 0;
  y = 0;

  analyse_cmd_line (argc, argv);

  change_gravity ();

  XMoveResizeWindow (dpy, win, x, y, w, h);

  XCloseDisplay (dpy);

  return 0;
}



static void
analyse_cmd_line_first (int argc, char **argv)
{
  int i;

  prog_name = argv[0];

  for (i = 1; i < argc; i++)
    {
      if (strcmp (argv[i], "-display") == 0 && i + 1 < argc)
	{
	  display_string = argv[++i];
	}
      else if (strcmp (argv[i], "-n_deco") == 0 && i + 1 < argc)
	{
	  n_deco = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-s_deco") == 0 && i + 1 < argc)
	{
	  s_deco = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-w_deco") == 0 && i + 1 < argc)
	{
	  w_deco = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-e_deco") == 0 && i + 1 < argc)
	{
	  e_deco = atoi (argv[++i]);
	}
    }

  if (n_deco == -1)
    n_deco = N_DECO;

  if (s_deco == -1)
    s_deco = S_DECO;

  if (w_deco == -1)
    w_deco = s_deco;

  if (e_deco == -1)
    e_deco = s_deco;
}

static void
analyse_cmd_line (int argc, char **argv)
{
  int i;

  for (i = 1; i < argc; i++)
    {
      if (strcmp (argv[i], "-display") == 0 && i + 1 < argc)
	{
	  i++;
	}
      else if (strcmp (argv[i], "-x") == 0 && i + 1 < argc)
	{
	  x = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-y") == 0 && i + 1 < argc)
	{
	  y = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-w") == 0 && i + 1 < argc)
	{
	  w = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-h") == 0 && i + 1 < argc)
	{
	  h = atoi (argv[++i]);
	}
      else if (strcmp (argv[i], "-geom") == 0 && i + 1 < argc)
	{
	  get_geometry (argv[++i], &x, &y, &w, &h);
	}
      else if (strcmp (argv[i], "-grav") == 0 && i + 1 < argc)
	{
	  grav_string = argv[++i];
	}
      else if (strcmp (argv[i], "-n_deco") == 0 && i + 1 < argc)
	{
	  i++;
	}
      else if (strcmp (argv[i], "-s_deco") == 0 && i + 1 < argc)
	{
	  i++;
	}
      else if (strcmp (argv[i], "-w_deco") == 0 && i + 1 < argc)
	{
	  i++;
	}
      else if (strcmp (argv[i], "-e_deco") == 0 && i + 1 < argc)
	{
	  i++;
	}
      else
	{
	  show_usage ();
	}
    }
}



static void
show_usage (void)
{
  fprintf (stderr, "%s : move and resize the focus window\n", prog_name);
  fprintf (stderr, "Usage : %s [OPTIONS]\n", prog_name);
  fprintf (stderr, "Where options are :\n");
  fprintf (stderr, "  -display        X server to contact\n");
  fprintf (stderr, "  -x              new x coordinate\n");
  fprintf (stderr, "  -y              new y coordinate\n");
  fprintf (stderr, "  -w              new width of window\n");
  fprintf (stderr, "  -h              new height of window\n");
  fprintf (stderr, "  -geom           new geometry of window\n");
  fprintf (stderr,
	   "  -grav           gravity of window (n, ne, e, se, s, sw, w, nw, c)\n");
  fprintf (stderr,
	   "  -n_deco         size of north (top) decoration on window\n");
  fprintf (stderr,
	   "  -s_deco         size of south (bottom) decoration on window\n");
  fprintf (stderr,
	   "  -w_deco         size of west (left) decoration on window\n");
  fprintf (stderr,
	   "  -e_deco         size of east (right) decoration on window\n");
  fprintf (stderr,
	   "                  (by default %s assume that w_deco=e_deco=s_deco)\n",
	   prog_name);

  exit (1);
}



static void
get_geometry (char *str_geom, int *x, int *y, int *width, int *height)
{
  char default_geometry[80];
  int bitmask;
  int px = *x, py = *y;
  int pw = *width, ph = *height;

  snprintf (default_geometry, 80, "%dx%d+%d+%d",
	   pw, ph, px, py);

  // Process the geometry specification
  bitmask = XGeometry (dpy, DefaultScreen (dpy),
		       str_geom,
		       default_geometry,
		       0,	/* bwidth */
		       1,	/* fwidth */
		       1,	/* fheight */
		       0,	/* xadder */
		       0,	/* yadder */
		       &px, &py,
		       &pw, &ph);

  // Check bitmask and set flags in XSizeHints structure
  if (bitmask & (WidthValue | HeightValue))
    {
      *width = pw;
      *height = ph;
    }

  if (bitmask & (XValue | YValue))
    {
      *x = px;
      *y = py;
    }
}



static void
change_gravity (void)
{
  if (strcmp (grav_string, "n") == 0)
    {
      x = (root_w - w - e_deco + w_deco) / 2 + y;
      y = y + n_deco;
    }
  else if (strcmp (grav_string, "ne") == 0)
    {
      x = root_w - x - w - e_deco;
      y = y + n_deco;
    }
  else if (strcmp (grav_string, "e") == 0)
    {
      x = root_w - x - w - e_deco;
      y = (root_h - h - s_deco + n_deco) / 2 + y;
    }
  else if (strcmp (grav_string, "se") == 0)
    {
      x = root_w - x - w - e_deco;
      y = root_h - y - h - s_deco;
    }
  else if (strcmp (grav_string, "s") == 0)
    {
      x = (root_w - w - e_deco + w_deco) / 2 + y;
      y = root_h - y - h - s_deco;
    }
  else if (strcmp (grav_string, "sw") == 0)
    {
      x = x + w_deco;
      y = root_h - y - h - s_deco;
    }
  else if (strcmp (grav_string, "w") == 0)
    {
      x = x + w_deco;
      y = (root_h - h - s_deco + n_deco) / 2 + y;
    }
  else if (strcmp (grav_string, "nw") == 0)
    {
      x = x + w_deco;
      y = y + n_deco;
    }
  else if (strcmp (grav_string, "c") == 0)
    {
      x = (root_w - w - e_deco + w_deco) / 2 + y;
      y = (root_h - h - s_deco + n_deco) / 2 + y;
    }
  else
    {
      x += w_deco;
      y += n_deco;
    }
}
